package com.jiuqi.util.type;

import java.util.Arrays;
import java.security.SecureRandom;
import java.math.BigInteger;
import com.jiuqi.util.hardware.HardwareUtil;
import java.util.Date;
import java.util.Random;

public final class GUID implements Cloneable, Comparable
{
    private static final char[] HEX_CHARS;
    private static final byte[] hardCode;
    private static final Random seedProvider;
    public static final String EMPTY = "00000000000000000000000000000000";
    private Random random;
    private byte[] code;
    private char[] buffer;
    private int hashcode;
    
    static {
        HEX_CHARS = new char[] { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
        final Date today = new Date();
        byte[] buf = HardwareUtil.getMachineSign("GUID");
        if (buf == null) {
            final BigInteger value = BigInteger.valueOf(today.getTime() ^ System.currentTimeMillis());
            final byte[] data = value.toByteArray();
            buf = new byte[8];
            System.arraycopy(data, 0, buf, 0, (data.length > 8) ? 8 : data.length);
        }
        hardCode = buf;
        (seedProvider = new SecureRandom()).setSeed(today.getTime());
    }
    
    public GUID() {
        synchronized (GUID.seedProvider) {
            this.random = new Random(GUID.seedProvider.nextLong());
            monitorexit(GUID.seedProvider);
        }
        this.code = new byte[16];
        this.buffer = new char[32];
        this.hashcode = 0;
    }
    
    public void generate() {
        final BigInteger v1 = BigInteger.valueOf(System.currentTimeMillis());
        byte[] data = v1.toByteArray();
        if (data.length >= 8) {
            System.arraycopy(data, 0, this.code, 0, 8);
        }
        else {
            System.arraycopy(data, 0, this.code, 0, data.length);
            System.arraycopy(GUID.hardCode, 0, this.code, data.length, 8 - data.length);
        }
        final BigInteger v2 = new BigInteger(64, this.random);
        data = v2.toByteArray();
        System.arraycopy(data, 0, this.code, 16 - data.length, data.length);
        for (int i = 0; i < 8 - data.length; ++i) {
            this.code[8 + i] = GUID.hardCode[8 - i - 1];
        }
        this.hashcode = 0;
    }
    
    public byte[] toBytes() {
        final byte[] data = new byte[this.code.length];
        System.arraycopy(this.code, 0, data, 0, this.code.length);
        return data;
    }
    
    @Override
    public String toString() {
        for (int i = 0; i < this.code.length; ++i) {
            this.buffer[i * 2] = GUID.HEX_CHARS[this.code[i] >> 4 & 0xF];
            this.buffer[i * 2 + 1] = GUID.HEX_CHARS[this.code[i] & 0xF];
        }
        return new String(this.buffer);
    }
    
    @Override
    public int hashCode() {
        int h = this.hashcode;
        if (h == 0) {
            for (int i = 0; i < this.code.length; ++i) {
                h = h * 31 + this.code[i];
            }
            this.hashcode = h;
        }
        return h;
    }
    
    @Override
    public boolean equals(final Object obj) {
        if (this == obj) {
            return true;
        }
        if (this == null || obj == null) {
            return false;
        }
        if (!(obj instanceof GUID)) {
            return false;
        }
        final GUID other = (GUID)obj;
        return this.hashCode() == other.hashCode() && Arrays.equals(this.code, other.code);
    }
    
    public Object clone() {
        try {
            final GUID result = (GUID)super.clone();
            result.code = new byte[16];
            System.arraycopy(this.code, 0, result.code, 0, 16);
            result.buffer = new char[32];
            System.arraycopy(this.buffer, 0, result.buffer, 0, 32);
            return result;
        }
        catch (final CloneNotSupportedException ex) {
            return null;
        }
    }
    
    public int compareTo(final Object arg0) {
        final byte[] data = ((GUID)arg0).code;
        for (int i = 0; i < 16; ++i) {
            final int ret = this.code[i] - data[i];
            if (ret != 0) {
                return ret;
            }
        }
        return 0;
    }
    
    public static String newGUID() {
        final GUID guid = new GUID();
        guid.generate();
        return guid.toString();
    }
}
